package com.zh.project0512.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.tokenValid;
import com.zh.project0512.model.QywxDepartmentUserLink;
import com.zh.project0512.model.User;
import com.zh.project0512.service.IQywxDepartmentService;
import com.zh.project0512.service.IQywxDepartmentUserLinkService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.HttpUtil;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

@Tag(name = "企业微信管理")
@RestController
@RequestMapping("/wxApp/qywx")
public class QYWXController {
    @Autowired
    private HttpUtil httpUtil;
    @Autowired
    private IQywxDepartmentService qywxDepartmentService;
    @Autowired
    private IUserService userService;
    @Resource
    private IQywxDepartmentUserLinkService qywxDepartmentUserLinkService;

    @Operation(summary = "刷新企业部门信息")
    @PostMapping("/departmentRefresh")
    public Result departmentRefresh() {
        String access_token = httpUtil.qywxGetToken();
        JSONObject res = httpUtil.qywxDepartmentList(access_token);
        List department = res.getJSONArray("department");
        qywxDepartmentService.remove(new QueryWrapper<>());
        qywxDepartmentService.updateAll(department);
        return Result.success(res.getJSONArray("department"));
    }

    @Operation(summary = "用户部门信息", parameters = {@Parameter(name = "code", description = "login返回的code（未使用过！）")})
    @PostMapping("/userDepartmentRefresh")
    @tokenValid
    public Result userDepartmentRefresh(@RequestBody @Parameter(hidden = true) JSONObject obj, @RequestHeader(value = "token") @Parameter(name = "登录token") String token) {
        String openid = new JwtUtil().parseOpenid(token);
        if (null == openid) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "无效token");
        }
        String access_token = httpUtil.qywxGetToken();
        JSONObject res = httpUtil.qywxUserSession(access_token, obj.getString("code"));
        String userid = res.getString("userid");
        JSONObject res1 = httpUtil.qywxUser(access_token, userid);
        Integer main_department = res1.getInteger("main_department");
        JSONArray department = res1.getJSONArray("department");

        if (null == userid || null == main_department) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "获取企业用户信息失败");
        }

        //先删除部门-用户关联项
        qywxDepartmentUserLinkService.delete(openid);
        //再添加部门-用户关联项
        if (department != null && department.size() >0){
            for (Object departmentId : department) {
                QywxDepartmentUserLink qywxDepartmentUserLink = new QywxDepartmentUserLink();
                qywxDepartmentUserLink.setOpenid(openid);
                qywxDepartmentUserLink.setDepartmentId((Integer) departmentId);
                qywxDepartmentUserLinkService.insert(qywxDepartmentUserLink);
            }
        }


        Integer sub_department = main_department;
        if (sub_department != null && sub_department != 1 && sub_department != 0) {
            sub_department = httpUtil.qywxDepartmentParentid(access_token, main_department, 0);
        }
        UpdateWrapper<User> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("openid", openid)
            .set("main_department", main_department).set("sub_department",sub_department).set("updateAt", LocalDateTime.now());
        userService.update(null, updateWrapper);
        return Result.success(res1);
    }

}
