package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zh.project0512.annotation.adminNeedNoTokenValid;
import com.zh.project0512.model.Admin;
import com.zh.project0512.model.User;
import com.zh.project0512.service.IAdminService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.*;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-18
 */
@Tag(name = "后台账号管理")
@RestController
@RequestMapping("manage/admin")
public class AdminController {

    @Autowired
    private IAdminService adminService;

    @Data
    static class LoginParam {
        @NotNull(message = "name不能为空")
        @Schema(title = "账号名称")
        private String name;
        @NotNull(message = "密码不能为空")
        @Schema(title = "密码")
        private String password;
    }

    @Operation(summary = "登录")
    @PostMapping("/login")
    @adminNeedNoTokenValid
    public Result login(@Validated @RequestBody LoginParam param) {
        String name = param.getName();
        String password = param.getPassword();
        QueryWrapper<Admin> qw = new QueryWrapper<>();
        qw.eq("name",name);
        Admin admin = adminService.getOne(qw);
        if(null == admin){
            return Result.fail(HttpStatusEnum.USER_NOT_EXIST, "未注册！");
        }
        if(!password.equals(admin.getPassword())){
            return Result.fail(HttpStatusEnum.USER_NOT_EXIST, "密码错误！");
        }
        String token = new JwtUtil().createAdminJWT(admin.getId().toString());
        admin.setToken(token).setLastLoginAt(LocalDateTime.now());
        adminService.updateById(admin);
        return Result.success(admin);
    }

//    @Operation(summary = "小程序注册", parameters = {
//        @Parameter(name = "code", description = "login返回的code（未使用过！）"),
//        @Parameter(name = "userInfo", description = "getUserInfo接口的iv和encryptedData"),
//        @Parameter(name = "phone", description = "getPhoneNumber接口的iv和encryptedData")
//    })
//    @PostMapping("register")
//    public Result register(@RequestBody @Parameter(hidden = true) JSONObject obj) {
//        JSONObject res = weChatUtil.auth(obj.getString("code"));
//        String openid = res.getString("openid");
//        if (null == openid) {
//            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "无效的openid");
//        }
//        if (null != userService.selByOpenid(openid)) {
//            return Result.fail(HttpStatusEnum.TEL_REGISTERED, "已注册");
//        }
//        String session_key = (String) res.get("session_key");
//        JSONObject userinfo = obj.getJSONObject("userinfo");
//        JSONObject phone = obj.getJSONObject("phone");
//        String info = CoderUtil.decrypt(session_key, userinfo.getString("iv"), userinfo.getString("encryptedData"));
//        String info2 = CoderUtil.decrypt(session_key, phone.getString("iv"), phone.getString("encryptedData"));
//        JSONObject j = JSONObject.parseObject(info);
//        JSONObject k = JSONObject.parseObject(info2);
//        if (null == j || null == k) {
//            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "session_key解析失败");
//        }
//        User u = new User();
//        u.setAvatar((String) j.get("avatarUrl"));
//        u.setName((String) j.get("nickName"));
//        u.setGender((Integer) j.get("gender"));
//        u.setOpenid(openid);
//        u.setTel(k.getString("phoneNumber"));
//        u.setToken(new JwtUtil().createJWT(openid));
////        u.setToken(new JwtUtils().generateToken(openid, new Date().getTime()));
//        u.setCreatAt(LocalDateTime.now());
//        userService.save(u);
//        return Result.success(u);
//    }

}
