package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.model.ReferenceGroup;
import com.zh.project0512.service.IReferenceGroupService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-24
 */
@Validated
@RestController
@RequestMapping("/manage/referenceGroup")
@Tag(name = "素材分组")
public class ReferenceGroupController {
    @Autowired
    private IReferenceGroupService referenceGroupService;

    @Data
    static class AddRGParam {
        @NotNull(message = "name不能为空")
        @Schema(title = "分组名")
        private String name;
        @NotNull(message = "封面不能为空")
        @Schema(title = "封面地址")
        private String coverUrl;
    }
    @Operation(summary = "新增分组")
    @PostMapping("/add")
    public Result add(@Validated @RequestBody AddRGParam param) {
        referenceGroupService.addReferenceGroup(new ReferenceGroup().setName(param.getName()).setCoverUrl(param.getCoverUrl()).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成！");
    }

    @Data
    static class DelRGParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "分组id")
        private Integer id;
    }
    @Operation(summary = "删除分组")
    @PostMapping("/del")
    public Result del(@Validated @RequestBody DelRGParam param) {
        return MybatisPlusUtil.sqlResult(referenceGroupService.removeById(param.getId()), "删除");
    }

    @Data
    static class UpdRGParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "分组id")
        private Integer id;
        @Schema(title = "分组名")
        private String name;
        @Schema(title = "封面地址")
        private String coverUrl;
    }
    @Operation(summary = "更新分组")
    @PostMapping("/upd")
    public Result upd(@Validated @RequestBody UpdRGParam param) {
        UpdateWrapper<ReferenceGroup> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", param.getId());
        ReferenceGroup referenceGroup = new ReferenceGroup().setName(param.getName()).setCoverUrl(param.getCoverUrl()).setUpdateAt(LocalDateTime.now());
        return MybatisPlusUtil.sqlResult(referenceGroupService.update(referenceGroup, updateWrapper), "修改");
    }

    @Operation(summary = "分组列表")
    @PostMapping("/list")
    public Result list(@RequestBody(required = false) @Parameter(hidden = true) JSONObject obj) {
        QueryWrapper<ReferenceGroup> qw = new QueryWrapper<>();
        qw.orderByAsc("sortWeight").orderByDesc("updateAt","creatAt");
        return Result.success(referenceGroupService.pageMaps(MybatisPlusUtil.SetPage(obj),qw));
    }

    @Data
    static class UpdBSParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "标签id")
        private Integer id;
        @NotNull(message = "排序权重不能为空")
        @Min(value = 1, message = "排序权重最小值为1")
        @Schema(title = "排序权重:数字越小排序靠前")
        private Integer sortWeight;
    }

    @Operation(summary = "修改标签排序")
    @PostMapping("/updSort")
    public Result updSort(@Validated @RequestBody TagController.UpdBSParam param) {
       ReferenceGroup referenceGroup = referenceGroupService.getById(param.getId());
        int origin = referenceGroup.getSortWeight();
        int current = param.getSortWeight();
        if(origin==current){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"权重无变化");
        }
        referenceGroupService.updSort(origin,current,origin>current);
        return Result.success("修改完成");
    }
}
