package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.zh.project0512.annotation.tokenValid;
import com.zh.project0512.controller.wxApp.TaskUController;
import com.zh.project0512.model.*;
import com.zh.project0512.model.validatedDemo.TaskValidGroup1;
import com.zh.project0512.model.validatedDemo.TaskValidGroup2;
import com.zh.project0512.service.*;
import com.zh.project0512.utils.CustomUtil;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.jsonwebtoken.Claims;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-21
 */
@Tag(name = "任务管理")
@Validated
@RestController
@RequestMapping("/task")
public class TaskController {
    @Autowired
    private IUserService userService;
    @Autowired
    private ITaskService taskService;
    @Autowired
    private IUserTaskService userTaskService;
    @Autowired
    private ITaskTagService taskTagService;
    @Autowired
    private ITaskBrandService taskBrandService;
    @Autowired
    private ITaskReferenceService taskReferenceService;
    @Autowired
    private IRewardRuleService rewardRuleService;
    @Autowired
    private IRewardRuleTemplateService rewardRuleTemplateService;
    @Autowired
    private ICustomerActionNoteService customerActionNoteService;

    @Operation(summary = "新增")
    @PostMapping("/add")
    public Result add(@Validated(TaskValidGroup1.class) @RequestBody Task task) {
        taskService.save(task);
        int id = task.getId();
        taskTagService.addGroup(task.getTagList(), id);
        taskReferenceService.addGroup(task.getReferenceList(), id);
        taskBrandService.addGroup(task.getBrandList(), id);
        return Result.success("添加完成");
    }

    @Data
    static class DelTaskParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
    }
    @Data
    static class AddTaskTagParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "tagId不能为空")
        @Min(value = 1, message = "tagId最小值为1")
        @Schema(title = "标签id")
        private Integer tagId;
    }

    @Operation(summary = "新增任务标签")
    @PostMapping("/addTag")
    public Result addTag(@Validated @RequestBody AddTaskTagParam param) {
        Integer taskId = param.getId();
        Integer tagId = param.getTagId();
        if (null != taskTagService.getMap(Wrappers.<TaskTag>query().lambda().eq(TaskTag::getTaskId, taskId).eq(TaskTag::getTagId, tagId))) {
            return Result.success("数据已存在");
        }
        taskTagService.saveOrUpdate(new TaskTag().setTaskId(taskId).setTagId(tagId));
        return Result.success("添加完成");
    }

    @Data
    static class AddTaskBrandParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "brandId不能为空")
        @Min(value = 1, message = "brandId最小值为1")
        @Schema(title = "品牌id")
        private Integer brandId;
    }

    @Operation(summary = "新增任务品牌")
    @PostMapping("/addBrand")
    public Result addBrand(@Validated @RequestBody AddTaskBrandParam param) {
        Integer taskId = param.getId();
        Integer brandId = param.getBrandId();
        if (null != taskBrandService.getMap(Wrappers.<TaskBrand>query().lambda().eq(TaskBrand::getTaskId, taskId).eq(TaskBrand::getBrandId, brandId))) {
            return Result.success("数据已存在");
        }
        taskBrandService.saveOrUpdate(new TaskBrand().setTaskId(taskId).setBrandId(brandId).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成");
    }

    @Data
    static class DelTaskTagParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "tagId不能为空")
        @Min(value = 1, message = "tagId最小值为1")
        @Schema(title = "标签id")
        private Integer tagId;
    }

    @Operation(summary = "删除任务标签")
    @PostMapping("/delTag")
    public Result delTag(@Validated @RequestBody DelTaskTagParam param) {
        return MybatisPlusUtil.sqlResult(taskTagService.remove(Wrappers.<TaskTag>query().lambda().eq(TaskTag::getTaskId, param.getId()).eq(TaskTag::getTagId, param.getTagId())), "删除");
    }

    @Data
    static class DelTaskBrandParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "brandId不能为空")
        @Min(value = 1, message = "brandId最小值为1")
        @Schema(title = "品牌id")
        private Integer brandId;
    }

    @Operation(summary = "删除任务品牌")
    @PostMapping("/delBrand")
    public Result delBrand(@Validated @RequestBody DelTaskBrandParam param) {
        return MybatisPlusUtil.sqlResult(taskBrandService.remove(Wrappers.<TaskBrand>query().lambda().eq(TaskBrand::getTaskId, param.getId()).eq(TaskBrand::getBrandId, param.getBrandId())), "删除");
    }

    @Data
    static class AddTaskReferenceParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "referenceId不能为空")
        @Min(value = 1, message = "referenceId最小值为1")
        @Schema(title = "素材Id")
        private Integer referenceId;
    }

    @Operation(summary = "新增任务素材")
    @PostMapping("/addReference")
    public Result addBrand(@Validated @RequestBody AddTaskReferenceParam param) {
        Integer taskId = param.getId();
        Integer referenceId = param.getReferenceId();
        if (null != taskReferenceService.getMap(Wrappers.<TaskReference>query().lambda().eq(TaskReference::getTaskId, taskId).eq(TaskReference::getReferenceId, referenceId))) {
            return Result.success("数据已存在");
        }
        taskReferenceService.saveOrUpdate(new TaskReference().setTaskId(taskId).setReferenceId(referenceId).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成");
    }

    @Data
    static class DelTaskReferenceParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "referenceId不能为空")
        @Min(value = 1, message = "referenceId最小值为1")
        @Schema(title = "素材id")
        private Integer referenceId;
    }

    @Operation(summary = "删除任务素材")
    @PostMapping("/delReference")
    public Result delTag(@Validated @RequestBody DelTaskReferenceParam param) {
        return MybatisPlusUtil.sqlResult(taskReferenceService.remove(Wrappers.<TaskReference>query().lambda().eq(TaskReference::getTaskId, param.getId()).eq(TaskReference::getReferenceId, param.getReferenceId())), "删除");
    }

    @Operation(summary = "更新")
    @PostMapping("/upd")
    public Result upd(@Validated(TaskValidGroup2.class) @RequestBody Task task) {
       task.setUpdateAt(LocalDateTime.now());
        return MybatisPlusUtil.sqlResult(taskService.updateById(task), "修改");
    }

    @Operation(summary = "删除")
    @PostMapping("/del")
    public Result del(@Validated @RequestBody DelTaskParam param) {
        return MybatisPlusUtil.sqlResult(taskService.removeById(param.getId()), "删除");
    }

    @Data
    static class ListTParam {
        private int pageNum;
        private int pageSize;
        @Schema(title="任务状态",description = "0未上线；1已上线；2已结束")
        private Integer status;
        @Schema(title = "月份区间")
        private Integer month;
        @Schema(title = "tagId数组")
        private List<Integer> tagIdList;
        @Schema(title = "brandId数组")
        private List<Integer> brandIdList;
        @Schema(title = "任务名称关键词搜索")
        private String keyword;
    }
    @Operation(summary = "列表")
    @PostMapping("/list")
    public Result list(@Validated @RequestBody ListTParam param){
        QueryWrapper<Reference> qw = new QueryWrapper<>();
        if (null != param.getStatus()) {
            qw.eq("t1.status", param.getStatus());
        }
        if (null != param.getMonth()) {
            qw.between("t1.creatAt", LocalDateTime.now().minusMonths(param.getMonth()),LocalDateTime.now());
        }
        if (null != param.getKeyword()) {
            qw.like("t1.title", param.getKeyword());
        }
        IPage iPage = taskService.idList(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()),param.getTagIdList(),param.getBrandIdList(), qw);
        List list = iPage.getRecords();
        if (list.size() > 0) {
            iPage.setRecords(taskService.list(null,list,param.getTagIdList(),param.getBrandIdList(), qw));
        }
        return Result.success(iPage);
    }

    @Data
    static class DetTaskParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("detail")
    public Result det(@Validated @RequestBody DetTaskParam param) {
        Task task = taskService.detailById(null, param.getId());
        List rewardRulesList = rewardRuleService.listByTemplateId(task.getRewardRuleTemplateId());
        JSONObject res = (JSONObject) JSONObject.toJSON(task);
        res.put("rewardRuleList", rewardRulesList);
        return Result.success(res);
    }

    @Operation(summary = "线索搜集列表")
    @PostMapping("/customerNoteList")
    public Result customerNote(@RequestBody JSONObject obj) {
        QueryWrapper<CustomerActionNote> qw = new QueryWrapper<>();
        String keywords = obj.getString("keywords");
        if(keywords != null) {
            qw.and(
                wrapper ->
                    wrapper.like("t1.customerName", keywords).or().like("t3.title", keywords).like("t1.tel", keywords)
                        .or().like("t1.area", keywords).or().like("t1.remarks", keywords).or().like("t1.brandList", keywords)
            );
        }
        IPage<Map> iPage = customerActionNoteService.pageList(MybatisPlusUtil.SetPage(obj),null,qw);
        List<Map> records = iPage.getRecords();
        for(Map m:records){
            if(m.get("brandList") !=null){
                m.replace("brandList",new CustomUtil().stringToList(m.get("brandList").toString()));
            }
        }
        return Result.success(iPage);
    }
}

