package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.zh.project0512.model.TopicActivity;
import com.zh.project0512.service.ITopicActivityService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-26
 */
@Tag(name = "动态")
@RestController
@RequestMapping("/topicActivity")
public class TopicActivityController {
    @Autowired
    private ITopicActivityService topicActivityService;

    @Data
    static class AddTopicParam {
        @Schema(title = "标题")
        @NotNull(message = "title不能为空")
        private String title;
        @Schema(title = "类型", description = "1视频；2图片；3文章")
        @NotNull(message = "type不能为空")
        private Integer type;
        @NotNull(message = "content不能为空")
        private String content;
        @Schema(title = "封面")
        private String cover;
    }

    @Operation(summary = "新增动态")
    @PostMapping("/add")
    public Result add(@Validated @RequestBody AddTopicParam param) {
        topicActivityService.save(new TopicActivity().setTitle(param.getTitle()).setType(param.getType()).setContent(param.getContent()).setCover(param.getCover()).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成");
    }

    @Data
    static class DelTopicParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
    }

    @Operation(summary = "删除动态")
    @PostMapping("/del")
    public Result del(@Validated @RequestBody DelTopicParam param) {
        return MybatisPlusUtil.sqlResult(topicActivityService.removeById(param.getId()), "删除");
    }

    @Data
    static class UpdTopicParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
        @Schema(title = "标题")
        private String title;
        @Schema(title = "内容")
        private String content;
        @Schema(title = "类型", description = "1视频；2图片；3文章")
        private Integer type;
        @Schema(title = "封面")
        private String cover;
    }

    @Operation(summary = "更新动态内容")
    @PostMapping("/upd")
    public Result upd(@Validated @RequestBody UpdTopicParam param) {
        return MybatisPlusUtil.sqlResult(topicActivityService.updateById(new TopicActivity().setId(param.getId()).setTitle(param.getTitle()).setType(param.getType()).setCover(param.getCover()).setContent(param.getContent()).setUpdateAt(LocalDateTime.now())), "修改");
    }

    @Operation(summary = "动态列表")
    @PostMapping("/list")
    public Result list(@RequestBody(required = false) JSONObject obj) {
        return Result.success(topicActivityService.pageMaps(MybatisPlusUtil.SetPage(obj)));
    }

    @Data
    static class DetTaskParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("/detail")
    public Result detail(@Validated @RequestBody TaskController.DetTaskParam param) {
        int id =param.getId();
        TopicActivity topicActivity = topicActivityService.getById(id);
        return topicActivity != null ? Result.success(topicActivity) : Result.fail(HttpStatusEnum.NOT_FOUND);
    }
}
